function [sys,x0,str,ts] = RLS(t,X,U,flag)
%
% RLS pour l'identification d'un ARX(2,2) i.e.
%    y(t)+a(1)y(t-1)+a(2)y(t-2)=b(1)u(t-1)+b(2)u(t-2)
%
switch flag,
  case 0
    [sys,x0,str,ts] = mdlInitializeSizes; % Initialization
  case 2
    sys = mdlUpdate(t,X,U); % Update discrete states
  case 3
    sys = mdlOutputs(t,X,U); % Calculate outputs
  case {1, 4, 9} % Unused flags
    sys = [];
  otherwise
    error(['unhandled flag = ',num2str(flag)]); % Error handling
end
% End of dsfunc.
%==============================================================
% Initialization
%==============================================================
function [sys,x0,str,ts] = mdlInitializeSizes
% Call simsizes for a sizes structure, fill it in, and convert it 
% to a sizes array.
sizes = simsizes;
sizes.NumContStates  = 0;
sizes.NumDiscStates  = 4+4+10;
sizes.NumOutputs     = 4;
sizes.NumInputs      = 2;
sizes.DirFeedthrough = 0; % Matrix D is empty.
sizes.NumSampleTimes = 1;
sys = simsizes(sizes); 
X0=zeros(4+4,1);
P0=1000*eye(4);
x0=[X0;SymReshape(P0)];   % Initialize the discrete states.
str = [];                 % Set str to an empty matrix.
ts  = [1 0];              % sample time: [period, offset]
% End of mdlInitializeSizes.
%==============================================================
% Update the discrete states
%==============================================================
function sys = mdlUpdate(t,X,U)
theta=X(1:4);
phi=X(5:8)';
P=SymReshape(X(9:18));
y=U(1);
u=U(2);
L=P*phi'/(0.1+phi*P*phi');
P=(eye(4)-L*phi)*P;
theta=theta+L*(y-phi*theta);
phi=[-y;phi(1);u;phi(3)];
sys = [theta;phi;SymReshape(P)];
% End of mdlUpdate.
%==============================================================
% Calculate outputs
%==============================================================
function sys = mdlOutputs(t,X,u)
sys = X(1:4);
% End of mdlOutputs.